/*
	PROforma, PROGS FOnt & Raster MAnager

	Show operators. Character display routines for PROforma.

	PFShow gstate, short *string

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 22 August 1993
	15.10.1995 use specific PFMoveR -> faster, init gstate->font here
	25.11.1995 current point was no longer advanced by show operators -> fixed
	27.11.1996 PFPShow & PFPWidth added
	03.08.1997 PFShow(Kern)Just now properly uses ExtraEos when counting spaces
	27.11.1997 PFShowTrack now properly uses ExtraEOS
*/

#include "gstate.h"
#include "system.h"
#include "cache.h"

#define MoveCursor	reluser2dev(gstate,w); \
					gstate->posdev.x+=w->x; \
					gstate->posdev.y+=w->y
#define Done		gstate->subpathorg=gstate->posdev; \
					return PFPathClear(gstate)

Error PFShow(Gstate gstate, const short *string)
{
	coor *w=&gstate->CharWidth;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string++);
		MoveCursor;
	}
	Done;
}

Error PFShowKern(Gstate gstate, const short *string)
{
	coor *w=&gstate->CharWidth;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string);
		KernAdjustWidth(gstate, *string, *(string+1));
		MoveCursor;
		string++;
	}
	Done;
}

Error PFShowX(Gstate gstate, const short *string, const pt *Xlist)
{
	coor *w=&gstate->CharWidth;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string++);
		w->x=*Xlist++;
		w->y=0;
		MoveCursor;
	}
	Done;
}

Error PFShowXY(Gstate gstate, const short *string, const pt *Xlist, const pt *Ylist)
{
	coor *w=&gstate->CharWidth;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string++);
		w->x=*Xlist++;
		w->y=*Ylist++;
		MoveCursor;
	}
	Done;
}

Error PFShowTrack(Gstate gstate, const short *string, pt width)
{
	coor *w=&gstate->CharWidth;
	const short *s=string;
	pt add=0, wdt; short len=0;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	/* determine length of string */
	for (s=string,len=0 ; StringContinues(gstate,*s) ; s++) len++;
	PFWidth(gstate,string,&wdt);
	if (len) add=(width-wdt)/(len-1);

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string++);
		w->x+=add;
		MoveCursor;
	}
	Done;
}

Error PFShowJust(Gstate gstate, const short *string, pt width)
{
	coor *w=&gstate->CharWidth;
	const short *s;
	pt add=0, wdt;
	int len;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	/* determine length of string */
	for (s=string,len=0 ; StringContinues(gstate,*s) ; s++) if (*s==' ') len++;
	PFWidth(gstate,string,&wdt);
	if (len) add=(width-wdt)/len;

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string);
		if (*string++==' ')
			w->x+=add;
		MoveCursor;
	}
	Done;
}

Error PFShowKernJust(Gstate gstate, const short *string, pt width)
{
	coor *w=&gstate->CharWidth;
	const short *s;
	pt add=0, wdt;
	int len;

	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	/* determine length of string */
	for (s=string,len=0 ; StringContinues(gstate,*s) ; s++) if (*s==' ') len++;
	PFWidthKern(gstate,string,&wdt);
	if (len) add=(width-wdt)/len;

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string);
		KernAdjustWidth(gstate, *string, *(string+1));
		if (*string++==' ')
			w->x+=add;
		MoveCursor;
	}
	Done;
}

Error PFPShow(Gstate gstate, const short *string)
{
	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	if (!gstate->font->mwidth)
	{
		gstate->noDisplayChars=TRUE;
		PFShowChar(gstate, 'm');
		gstate->noDisplayChars=FALSE;
	}

	while (StringContinues(gstate,*string))
	{
		PFShowChar(gstate, *string++);
		gstate->posdev.x+=gstate->chwx;
	}
	Done;
}

Error PFPWidth(Gstate gstate, const short *string, short *xwidth)
{
	short width=0;
	*xwidth=0;
	if (!string) return ERR_OK;
	if (!gstate->font) CacheFontFind(gstate);

	while (StringContinues(gstate,*string))
		width+=PCharWidth(gstate, *string++);
	*xwidth=width;
	return ERR_OK;
}
