/*
	PROforma, PROgs FOnt & Raster MAnager

	fixdiv
	fixsqrt
		routines to calculate with fixpoint numbers
		This code is based on METAFONT, para 114 and 121
	fixdist 	fixdist(a,b)=fixsqrt( fixmul(a,a)+fixmul(b,b) )

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	03.12.1996 started
	10.03.1997 fixdist added
*/

#include <sms.h>
#include "PROforma.h"

pt fixdiv(pt p, pt q)
{
	int f;							/* faction bits with leading 1 bit */
	int n;							/* integer part of |p/q| */
	char negative=FALSE;			/* should result be negated ? */
	int be_careful; 				/* disables certain compiler optimizations */

	if (p<0) { p=-p; negative=TRUE; }
	if (q<=0)
	{
		if (q==0) { /* DIVIDE BY ZERO */ q/=0; return ptmax; }
		q=-q;
		negative=!negative;
	}
	n=p/q;
	p=p%q;
	if (n>=0x8000)
	{
		/* OVERFLOW */
		return negative?ptmin:ptmax;
	}
	else
	{
		n=short2pt(n-1);
		f=1;
		do {
			be_careful=p-q;
			p=be_careful+p;
			if
				(p>=0) f+=f+1;
			else
				f+=f, p=p+q;
		} while (f<pt_one);
		be_careful=p-q;
		if (be_careful+p>=0) f++;

		return negative ? -(f+n) : f+n;
	}
}

#define floor	0x20000000
#define ceiling 0x40000000

pt fixsqrt(pt x)
{
	int k=23;						/* iteration control counter */
	int y, q=2; 					/* registers for intermediate results */

	if (x<=0) { /* if <0 ERROR */ return 0; }
	while (x<floor) k--, x*=4;
	if (x<ceiling)
		y=0;
	else
		x=x-ceiling, y=1;
	do {
		x+=x; y+=y;
		if (x>=ceiling) x=x-ceiling, y++;
		x+=x; y+=y-q; q+=q;
		if (x>=ceiling) x=x-ceiling, y++;
		if (y>q)
			y-=q, q+=2;
		else
			if (y<=0)
				q-=2, y+=q;
		k--;
	} while (k>0);
	return q/2;
}

pt fixdist(pt a, pt b)
{
	/* have to make sure that no overflow occurs */
	/* this allows lengths up to 1444mm, more than enough */
	/* but we also have to make sure that no underflow occurs !! */
#define square(x) fixmul(x,x)
#define ssquare(x) fixmul(x/64,x/64)
	if (abs(a)>short2pt(100) || abs(b)>short2pt(100))
		return fixsqrt( ssquare(a)+ssquare(b) )*64;
	else
		return fixsqrt( square(a)+square(b) );
}
