/*
	PROforma, PROGS Font & Raster Manager

	test program for v1.10
	check the speed of - fontcache
					   - mask drawing
					   - positioning strings
	The timing results are written to "ram1_cachetime_txt" (overwrite).
	test test - just use one font and one size to test the times spent drawing
	or searching the character to draw only. No switches between fonts etc.

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 5 October 1995
*/

#include "io.h"
#include "sms.h"
#include "job.h"
#include "time.h"
#include "thing.h"

#include "PROforma.h"

Bbox bbox={short2pt(720), short2pt(540),0,0};
ColourRGB colour={ 0, 0, 0 };

Error TimeDrawing(Gstate,Channel, int count, char *text, char show);

Error init()
{
	Gstate gstate;
	Error err;
	Channel chid;

	err=IOOpen("ram1_cachetime_txt",OPEN_OVER,&chid);
	if (err) return err;

	LinkPROforma();
	err=PFGstateInit(PF_DRIVER_SCREEN,NULL,&bbox,&gstate,NULL);
	if (err) return err;

	err=PFColourRGB(gstate,&colour);
	if (err) return err;

	err=PFFontLoad(gstate,"Courier");
	if (err) return err;
	PFFontScale(gstate,short2pt(12));

	err=PFPageShow(gstate);
	if (err) return err;
				 

	/* draw once to fill the fontcache */
	TimeDrawing(gstate,chid,1,"fill the fontcache",TRUE);

	/* the eye also wants something */
	err=PFPageShow(gstate);
	if (err) return err;

	/* test the speed of the fontcache */
	TimeDrawing(gstate,chid,500,"draw some text",TRUE);

	/* determine size of overhead (no PFShow Operators) */
	TimeDrawing(gstate,chid,500,"check overhead",FALSE);

	/* use dynamic config to replace bitmap draw by nop routintes */
	/* to determine which part of the time is used by bitmap driver */
	err=THINGCall(PF_THING_NAME,PF_THING_CNFG,1,"D mode4NoDraw_pfd");
	err=PFGstateInit(PF_DRIVER_SCREEN,NULL,&bbox,&gstate,NULL);
	if (err) return err;
	err=PFFontLoad(gstate,"Courier");
	if (err) return err;
	PFFontScale(gstate,short2pt(12));
	TimeDrawing(gstate,chid,500,"draw some text without drawing",TRUE);


	return err;
}

Error TimeDrawing(Gstate gstate,Channel chid,int count,char *text,char show)
{
	char str[80],time[30];
	Time before,after;

	IOPutString(chid,"third cache test, one font at screen size",NULL);
	STRFormatPut(str,"%s, repeated %d times",text,count);
	IOPutString(chid,str,NULL);
	TIMEGet(&before);
	for (;count>0;count--)
	{
		PFMoveTo(gstate, pt_hundred, short2pt(80));
		if (show) PFStringShow(gstate,"I just need to display a load of text to test all this.");
		PFMoveTo(gstate, pt_hundred, short2pt(120));
		if (show) PFStringShow(gstate,"I should be able to use the results to make it faster.");

		PFMoveTo(gstate, pt_hundred, short2pt(180));
		if (show) PFStringShow(gstate,"I just need to display a load of text to test all this.");
		PFMoveTo(gstate, pt_hundred, short2pt(220));
		if (show) PFStringShow(gstate,"I should be able to use the results to make it faster.");

		PFMoveTo(gstate, pt_hundred, short2pt(280));
		if (show) PFStringShow(gstate,"I just need to display a load of text to test all this.");
		PFMoveTo(gstate, pt_hundred, short2pt(320));
		if (show) PFStringShow(gstate,"I should be able to use the results to make it faster.");

		PFMoveTo(gstate, pt_hundred, short2pt(380));
		if (show) PFStringShow(gstate,"I just need to display a load of text to test all this.");
		PFMoveTo(gstate, pt_hundred, short2pt(420));
		if (show) PFStringShow(gstate,"I should be able to use the results to make it faster.");
	}
	TIMEGet(&after);
	STRTime(before,time);
	STRFormatPut(str,"before %s",time);
	IOPutString(chid,str,NULL);
	STRTime(after,time);
	STRFormatPut(str,"after %s",time);
	IOPutString(chid,str,NULL);
	STRFormatPut(str,"time used %d seconds",after-before);
	IOPutString(chid,str,NULL);
	return ERR_OK;
}
